@@ -6,18 +6,19 @@ import tempfile
 
 
 class TodoTxt:
-    def __init__(self, filename):
+    def __init__(self, filename, encoding='utf-8'):
         self.filename = pathlib.Path(filename)
+        self.encoding = encoding
         self.tasks = []
 
     def parse(self):
         self.tasks = []
 
-        with open(self.filename, 'rt') as fd:
+        with open(self.filename, 'rt', encoding=self.encoding) as fd:
             for linenr, line in enumerate(fd.readlines()):
                 if len(line.strip()) == 0:
                     continue
-                task = Task(line, linenr=linenr)
+                task = Task(line, linenr=linenr, todotxt=self)
                 self.tasks.append(task)
 
         return self.tasks
@@ -29,19 +30,22 @@ class TodoTxt:
             target = pathlib.Path(target)
         write_to = target
 
+        tmpfile = None
         if safe:
-            tmpfile = tempfile.NamedTemporaryFile(dir=self.filename.parent, delete=False)
+            tmpfile = tempfile.NamedTemporaryFile(dir=self.filename.parent)
             write_to = tmpfile.name
-            tmpfile.close()
 
         with open(write_to, 'wb', buffering=0) as fd:
             lines = [str(task) + '\r\n' for task in
                      sorted(self.tasks, key=lambda t: t.linenr if t.linenr is not None else len(self.tasks))]
-            fd.write(bytes(''.join(lines), 'utf-8'))
+            fd.write(bytes(''.join(lines), self.encoding))
 
         if safe:
             os.replace(write_to, target)
 
+        if tmpfile is not None:
+            tmpfile.close()
+
 
 class Task:
     COMPLETED_RE = re.compile(r'^x\s+')
@@ -52,7 +56,7 @@ class Task:
     DATE_RE = re.compile(r'^\s*([\d]{4}-[\d]{2}-[\d]{2})', re.ASCII)
     DATE_FMT = '%Y-%m-%d'
 
-    def __init__(self, line=None, linenr=None):
+    def __init__(self, line=None, linenr=None, todotxt=None):
         self.description = None
         self.is_completed = None
         self.priority = None
@@ -60,6 +64,8 @@ class Task:
         self.creation_date = None
         self.linenr = linenr
         self.raw = None
+        self.todotxt = todotxt
+        self._attributes = None
 
         if len(line.strip()) > 0:
             self.parse(line)
@@ -88,7 +94,7 @@ class Task:
                 if value is None or match.group(3) == value:
                     start, end = match.span()
                     self.description = self.description[:start]
-                    self.raw = str(self)
+                    self.parse(str(self))
                     success = True
 
                 break
@@ -103,7 +109,7 @@ class Task:
             if match.group(2) == text:
                 start, end = match.span()
                 self.description = self.description[:start] + self.description[end:]
-                self.raw = str(self)
+                self.parse(str(self))
                 return True
         return False
 
@@ -116,7 +122,7 @@ class Task:
             self.description = self.description[:match.start(3)] + \
                                newvalue + \
                                self.description[match.end(3):]
-            self.raw = str(self)
+            self.parse(str(self))
             return True
 
         return False
@@ -135,20 +141,21 @@ class Task:
                 self.description = self.description[:match.start(2)] + \
                                    newvalue + \
                                    self.description[match.end(2):]
+                self.parse(str(self))
                 return True
         return False
 
     def add_project(self, project):
         self.description += ' +' + project
-        self.raw = str(self)
+        self.parse(str(self))
 
     def add_context(self, context):
         self.description += ' @' + context
-        self.raw = str(self)
+        self.parse(str(self))
 
     def add_attribute(self, key, value):
         self.description += f' {key}:{value}'
-        self.raw = str(self)
+        self.parse(str(self))
 
     @property
     def projects(self):
@@ -160,19 +167,34 @@ class Task:
 
     @property
     def attributes(self):
-        result = {}
+        if self._attributes is None:
+            self.parse_attributes()
+        return self._attributes
+
+    def __getattr__(self, name, fallback=None):
+        if name.startswith('attr_'):
+            if fallback is None:
+                fallback = []
+            _, attrname = name.split('_', 1)
+            return self.attributes.get(attrname, fallback)
+        return AttributeError(name)
+
+    def parse_attributes(self):
+        self._attributes = {}
         for match in self.parse_tags(Task.KEYVALUE_RE):
             key = match.group(2)
             value = match.group(3)
             if key in ['http', 'https', 'mailto', 'ssh', 'ftp']:
                 continue
-            if key not in result:
-                result[key] = []
-            result[key].append(value)
-        return result
+            if key not in self._attributes:
+                self._attributes[key] = []
+            self._attributes[key].append(value)
 
     def parse_tags(self, regex):
         matches = []
+        if self.description is None:
+            return matches
+
         for match in regex.finditer(self.description):
             if match:
                 matches.append(match)
@@ -209,6 +231,8 @@ class Task:
         if len(line) > 0:
             self.description = line.strip()
 
+        self.parse_attributes()
+
     def __str__(self):
         result = ''
         if self.is_completed:
